"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Logger = void 0;
const api_1 = require("./api");
const utils_1 = require("@ztimson/utils");
class Logger {
    api;
    constructor(api, logLevel) {
        this.api = typeof api == 'string' ? new api_1.Api(api) : api;
        if (logLevel != null && logLevel != 'NONE') {
            // LOG_LEVEL[logLevel];
            window.addEventListener('error', (event) => this.error(event.error.stack));
            window.addEventListener('unhandledrejection', async (event) => this.error(event.reason.stack));
        }
    }
    buildLog(level, log) {
        return {
            time: new Date().getTime(),
            level,
            log,
            ctx: {
                cores: navigator.hardwareConcurrency,
                mem: navigator?.deviceMemory,
                res: [window.innerWidth, window.innerHeight],
                url: location.href,
            }
        };
    }
    clearClientLogs() {
        return this.api.request({ url: `/api/logs/client`, method: 'DELETE' });
    }
    clearServerLogs() {
        return this.api.request({ url: `/api/logs/server`, method: 'DELETE' });
    }
    clientLogs(length, page) {
        const query = [length ? `length=${length}` : undefined, page ? `page=${page}` : undefined,].filter(v => !!v).join('&');
        return this.api.request({ url: `/api/logs/client${query ? `?${query}` : ''}` });
    }
    serverLogs(length, page) {
        const query = [length ? `length=${length}` : undefined, page ? `page=${page}` : undefined,].filter(v => !!v).join('&');
        return this.api.request({ url: `/api/logs/server${query ? `?${query}` : ''}` });
    }
    debug(...logs) {
        return this.api.request({ url: `/api/logs/client`, body: this.buildLog(utils_1.LOG_LEVEL.DEBUG, logs) }).catch(() => { });
    }
    log(...logs) {
        return this.api.request({ url: `/api/logs/client`, body: this.buildLog(utils_1.LOG_LEVEL.LOG, logs) }).catch(() => { });
    }
    info(...logs) {
        return this.api.request({ url: `/api/logs/client`, body: this.buildLog(utils_1.LOG_LEVEL.INFO, logs) }).catch(() => { });
    }
    warn(...logs) {
        return this.api.request({ url: `/api/logs/client`, body: this.buildLog(utils_1.LOG_LEVEL.WARN, logs) }).catch(() => { });
    }
    error(...logs) {
        return this.api.request({ url: `/api/logs/client`, body: this.buildLog(utils_1.LOG_LEVEL.ERROR, logs) }).catch(() => { });
    }
}
exports.Logger = Logger;
