"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Momentum = void 0;
const actions_1 = require("./actions");
const api_1 = require("./api");
const auth_1 = require("./auth");
const data_1 = require("./data");
const email_1 = require("./email");
const groups_1 = require("./groups");
const utils_1 = require("@ztimson/utils");
const logger_1 = require("./logger");
const pdf_1 = require("./pdf");
const sockets_1 = require("./sockets");
const storage_1 = require("./storage");
const users_1 = require("./users");
const settings_1 = require("./settings");
const static_1 = require("./static");
class Momentum extends utils_1.TypedEmitter {
    api;
    actions;
    auth;
    data;
    email;
    groups;
    logger;
    pdf;
    settings;
    socket;
    static;
    storage;
    users;
    constructor(url, opts) {
        super();
        this.api = new api_1.Api(url, opts?.api);
        this.actions = new actions_1.Actions(this.api);
        this.auth = new auth_1.Auth(this.api, {
            persist: opts?.persist ?? true,
            loginUi: opts?.loginUi
        });
        this.data = new data_1.Data(this.api);
        this.email = new email_1.Email(this.api);
        this.groups = new groups_1.Groups(this.api);
        this.logger = new logger_1.Logger(this.api, opts?.logLevel);
        this.pdf = new pdf_1.Pdf(this.api);
        this.settings = new settings_1.Settings(this.api);
        this.static = new static_1.Static(this.api);
        this.storage = new storage_1.Storage(this.api);
        this.users = new users_1.Users(this.api);
        if (opts?.socket)
            this.socket = new sockets_1.Socket(this.api);
        this.api.on('*', (event, ...args) => this.emit(`API::${event}`, ...args));
        this.actions.on('*', (event, ...args) => this.emit(`ACTIONS::${event}`, ...args));
        this.auth.on('*', (event, ...args) => this.emit(`AUTH::${event}`, ...args));
        this.data.on('*', (event, ...args) => this.emit(`DATA::${event}`, ...args));
        this.email.on('*', (event, ...args) => this.emit(`EMAIL::${event}`, ...args));
        this.groups.on('*', (event, ...args) => this.emit(`GROUPS::${event}`, ...args));
        this.pdf.on('*', (event, ...args) => this.emit(`PDF::${event}`, ...args));
        this.settings.on('*', (event, ...args) => this.emit(`SETTINGS::${event}`, ...args));
        this.static.on('*', (event, ...args) => this.emit(`STATIC::${event}`, ...args));
        this.storage.on('*', (event, ...args) => this.emit(`STORAGE::${event}`, ...args));
        this.users.on('*', (event, ...args) => this.emit(`USERS::${event}`, ...args));
        this.users.on('*', (event, ...args) => {
            if (Array.isArray(args[0])) {
                const u = args[0].find(u => u._id == this.auth.user?._id);
                if (u)
                    this.auth.user = u;
            }
            else if (args[0]._id == this.auth.user?._id) {
                this.auth.user = args[0];
            }
        });
    }
}
exports.Momentum = Momentum;
