"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Socket = void 0;
const api_1 = require("./api");
class Socket {
    static timeout = 10000;
    api;
    url;
    connection;
    open = false;
    constructor(api) {
        this.api = typeof api == 'string' ? new api_1.Api(api) : api;
        this.url = this.api.url.replace('http', 'ws');
        this.api.on('TOKEN', () => this.connect());
        if (!this.api.token)
            this.connect();
    }
    close() {
        console.debug('Disconnected from Momentum');
        this.open = false;
        this.connection.close();
    }
    connect(retry = 3) {
        if (this.connection?.readyState < 2)
            this.connection.close();
        this.connection = new WebSocket(this.url + (this.api.token ? `?token=${this.api.token}` : ''));
        const timeout = setTimeout(() => {
            if (this.open)
                return;
            this.connection.close();
            console.error(`Momentum connection timeout`);
            if (retry > 0)
                this.connect(retry - 1);
        }, Socket.timeout);
        this.connection.onclose = () => this.open = false;
        this.connection.onmessage = this.handle;
        this.connection.onopen = () => {
            this.open = true;
            clearTimeout(timeout);
            console.debug('Connected to Momentum');
        };
    }
    handle(...args) {
        console.log(args);
    }
    send(channel, payload) {
        this.connection.send(JSON.stringify({
            token: this.api.token,
            channel,
            payload
        }));
    }
}
exports.Socket = Socket;
