"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Storage = void 0;
const api_1 = require("./api");
const utils_1 = require("@ztimson/utils");
class Storage extends utils_1.TypedEmitter {
    api;
    constructor(api) {
        super();
        this.api = typeof api == 'string' ? new api_1.Api(api) : api;
    }
    delete(path) {
        const url = (path.startsWith('/api/storage/') ? path : '/api/storage/' + path).replaceAll('//', '/');
        return this.api.request({ url, method: 'DELETE' }).then(() => {
            this.emit('DELETE', url);
        });
    }
    list(path) {
        const url = (path.startsWith('/api/storage/') ? path : '/api/storage/' + path).replaceAll('//', '/');
        return this.api.request({ url: url + '?list' }).then(resp => {
            this.emit('LIST', path, resp);
            return resp;
        });
    }
    open(path, target = '_blank') {
        const p = (path.startsWith('/api/storage/') ? path : '/api/storage/' + path).replaceAll(/\/{2,}/g, '/');
        const link = `${this.api.url}${p}${this.api.token ? `?token=${this.api.token}` : ''}`;
        if (!target)
            return link;
        this.emit('OPEN', path);
        return window.open(link, target);
    }
    mkdir(path) {
        const p = (path.startsWith('/api/storage/') ? path : '/api/storage/' + path).replaceAll(/\/{2,}/g, '/');
        return this.api.request({ url: p + '?directory', method: 'POST' });
    }
    download(path, opts = {}) {
        return new utils_1.PromiseProgress((res, rej, progress) => {
            const p = ('/api/storage/' + path).replaceAll('//', '/');
            this.api.request({ ...opts, url: p, skipConverting: true }).then((response) => {
                if (!response.ok)
                    return rej(response.statusText);
                const contentLength = response.headers.get('Content-Length');
                const total = contentLength ? parseInt(contentLength, 10) : 0;
                let chunks = [], loaded = 0;
                const reader = response.body?.getReader();
                const process = (result) => {
                    if (result.done) {
                        progress(1);
                        const blob = new Blob(chunks);
                        const name = opts.downloadAs || new URL(response.url).pathname.split('/').pop();
                        const url = URL.createObjectURL(blob);
                        (0, utils_1.download)(url, name.includes('.') ? name : name + '.zip');
                        URL.revokeObjectURL(url);
                        this.emit('DOWNLOAD', path, blob);
                        res(blob);
                    }
                    else {
                        const chunk = result.value;
                        chunks.push(chunk);
                        loaded += chunk.length;
                        progress(loaded / total);
                        reader.read().then(process);
                    }
                };
                reader?.read().then(process);
            });
        });
    }
    async upload(files, opts = '') {
        const fileBrowser = () => {
            return new Promise(res => {
                const input = document.createElement('input');
                input.type = 'file';
                input.accept = opts?.accept || '*';
                input.style.display = 'none';
                input.multiple = !!opts?.multiple;
                input.onblur = input.onchange = async () => {
                    res(Array.from(input.files));
                    input.remove();
                };
                document.body.appendChild(input);
                input.click();
            });
        };
        if (!files)
            files = await fileBrowser();
        if (!files || Array.isArray(files) && !files.length)
            return [];
        const data = new FormData();
        const p = ('/api/storage/' + (typeof opts == 'string' ? opts : opts?.path)).replaceAll('//', '/');
        (Array.isArray(files) ? files : [files]).forEach(f => data.append('file', f));
        return this.api.request({ url: p, body: data }).then(resp => {
            this.emit('UPLOAD', resp);
            return resp;
        });
    }
}
exports.Storage = Storage;
